@@ -26,6 +26,12 @@ private Q_SLOTS:
     void testLineRemoved();
     void testLineWrapOrUnwrapUpdateRangeForLineCache();
     void testMultiline();
+    void testMultiblock();
+    void benchCursorsInsertionRemoval();
+    void benchCheckValidity_data();
+    void benchCheckValidity();
+    void benchRangeForLine();
+    void testMultiblockRangeWithLineUnwrapping();
 };
 
 #endif // KATE_MOVINGRANGE_TEST_H
