@@ -22,7 +22,6 @@ TextBlock::TextBlock(TextBuffer *buffer, int index)
 TextBlock::~TextBlock()
 {
     // blocks should be empty before they are deleted!
-    Q_ASSERT(m_blockSize == 0);
     Q_ASSERT(m_lines.empty());
     Q_ASSERT(m_cursors.empty());
 
@@ -56,13 +55,11 @@ void TextBlock::setLineMetaData(int line, const TextLine &textLine)
 void TextBlock::appendLine(const QString &textOfLine)
 {
     m_lines.emplace_back(textOfLine);
-    m_blockSize += textOfLine.size();
 }
 
 void TextBlock::clearLines()
 {
     m_lines.clear();
-    m_blockSize = 0;
 }
 
 void TextBlock::text(QString &text) const
@@ -192,6 +189,9 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         m_lines[0] = previousBlock->m_lines.back();
         previousBlock->m_lines.erase(previousBlock->m_lines.begin() + (previousBlock->lines() - 1));
 
+        m_buffer->m_blockSizes[m_blockIndex - 1] -= m_lines[0].length() + 1;
+        m_buffer->m_blockSizes[m_blockIndex] += m_lines[0].length();
+
         const int oldSizeOfPreviousLine = m_lines[0].text().size();
         if (oldFirst.length() > 0) {
             // append text
@@ -219,7 +219,7 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
 
         // move all cursors because of the unwrapped line
         // remember all ranges modified, optimize for the standard case of a few ranges
-        QVarLengthArray<TextRange *, 32> changedRanges;
+        QVarLengthArray<QPair<TextRange *, bool>, 32> changedRanges;
         for (TextCursor *cursor : m_cursors) {
             // this is the unwrapped line
             if (cursor->lineInBlock() == 0) {
@@ -230,7 +230,7 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
                 auto range = cursor->kateRange();
                 if (range && !range->isValidityCheckRequired()) {
                     range->setValidityCheckRequired();
-                    changedRanges.push_back(range);
+                    changedRanges.push_back({range, false});
                 }
             }
         }
@@ -239,15 +239,18 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         for (auto it = previousBlock->m_cursors.begin(); it != previousBlock->m_cursors.end();) {
             auto cursor = *it;
             if (cursor->lineInBlock() == lastLineOfPreviousBlock) {
+                Kate::TextRange *range = cursor->kateRange();
+                // get the value before changing the block
+                const bool spansMultipleBlocks = range && range->spansMultipleBlocks();
                 cursor->m_line = 0;
                 cursor->m_block = this;
-                m_cursors.insert(cursor);
+                insertCursor(cursor);
 
                 // remember range, if any, avoid double insert
-                auto range = cursor->kateRange();
                 if (range && !range->isValidityCheckRequired()) {
                     range->setValidityCheckRequired();
-                    changedRanges.push_back(range);
+                    // the range might not span multiple blocks anymore
+                    changedRanges.push_back({range, spansMultipleBlocks});
                 }
 
                 // remove from previous block
@@ -261,7 +264,11 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         // fixup the ranges that might be effected, because they moved from last line to this block
         // we might need to invalidate ranges or notify about their changes
         // checkValidity might trigger delete of the range!
-        for (TextRange *range : std::as_const(changedRanges)) {
+        for (auto [range, wasMultiblock] : changedRanges) {
+            // if the range doesn't span multiple blocks anymore remove it from buffer multiline range cache
+            if (!range->spansMultipleBlocks() && wasMultiblock) {
+                m_buffer->removeMultilineRange(range);
+            }
             // afterwards check validity, might delete this range!
             range->checkValidity();
         }
@@ -270,6 +277,8 @@ void TextBlock::unwrapLine(int line, TextBlock *previousBlock, int fixStartLines
         return;
     }
 
+    m_buffer->m_blockSizes[m_blockIndex] -= 1;
+
     // easy: just move text to previous line and remove current one
     const int oldSizeOfPreviousLine = m_lines.at(line - 1).length();
     const int sizeOfCurrentLine = m_lines.at(line).length();
@@ -355,8 +364,6 @@ void TextBlock::insertText(const KTextEditor::Cursor position, const QString &te
     // notify the text history
     m_buffer->history().insertText(position, text.size(), oldLength);
 
-    m_blockSize += text.size();
-
     // cursor and range handling below
 
     // no cursors in this block, no work to do..
@@ -431,8 +438,6 @@ void TextBlock::removeText(KTextEditor::Range range, QString &removedText)
     // notify the text history
     m_buffer->history().removeText(range, oldLength);
 
-    m_blockSize -= removedText.size();
-
     // cursor and range handling below
 
     // no cursors in this block, no work to do..
@@ -498,8 +503,9 @@ void TextBlock::splitBlock(int fromLine, TextBlock *newBlock)
     newBlock->m_lines.reserve(linesOfNewBlock);
     for (size_t i = fromLine; i < m_lines.size(); ++i) {
         auto line = std::move(m_lines[i]);
-        m_blockSize -= line.length();
-        newBlock->m_blockSize += line.length();
+        const int lineLength = line.length();
+        m_buffer->m_blockSizes[m_blockIndex] -= lineLength + 1;
+        m_buffer->m_blockSizes[newBlock->m_blockIndex] += lineLength + 1;
         newBlock->m_lines.push_back(std::move(line));
     }
 
@@ -514,7 +520,7 @@ void TextBlock::splitBlock(int fromLine, TextBlock *newBlock)
             cursor->m_block = newBlock;
 
             // add to new, remove from current
-            newBlock->m_cursors.insert(cursor);
+            newBlock->m_cursors.push_back(cursor);
             it = m_cursors.erase(it);
             if (cursor->kateRange()) {
                 ranges.insert(cursor->kateRange());
@@ -524,6 +530,8 @@ void TextBlock::splitBlock(int fromLine, TextBlock *newBlock)
             ++it;
         }
     }
+    // sort the cursors
+    std::sort(newBlock->m_cursors.begin(), newBlock->m_cursors.end());
 
     for (auto range : std::as_const(ranges)) {
         if (range->spansMultipleBlocks()) {
@@ -539,12 +547,13 @@ void TextBlock::mergeBlock(TextBlock *targetBlock)
     for (TextCursor *cursor : m_cursors) {
         cursor->m_line = cursor->lineInBlock() + targetBlock->lines();
         cursor->m_block = targetBlock;
-        targetBlock->m_cursors.insert(cursor);
+        targetBlock->m_cursors.push_back(cursor);
         if (cursor->kateRange()) {
             ranges.insert(cursor->kateRange());
         }
     }
     m_cursors.clear();
+    std::sort(targetBlock->m_cursors.begin(), targetBlock->m_cursors.end());
 
     for (auto range : std::as_const(ranges)) {
         if (!range->spansMultipleBlocks()) {
@@ -557,7 +566,6 @@ void TextBlock::mergeBlock(TextBlock *targetBlock)
     for (size_t i = 0; i < m_lines.size(); ++i) {
         targetBlock->m_lines.push_back(m_lines.at(i));
     }
-    targetBlock->m_blockSize += m_blockSize;
     clearLines();
 }
 
@@ -595,7 +603,7 @@ void TextBlock::clearBlockContent(TextBlock *targetBlock)
             cursor->m_column = 0;
             cursor->m_line = 0;
             cursor->m_block = targetBlock;
-            targetBlock->m_cursors.insert(cursor);
+            targetBlock->m_cursors.push_back(cursor);
 
             // remove it and advance to next element
             it = m_cursors.erase(it);
