@@ -10,8 +10,6 @@
 #include "katetextline.h"
 
 #include <QList>
-#include <QSet>
-#include <QVarLengthArray>
 
 #include <ktexteditor/cursor.h>
 #include <ktexteditor_export.h>
@@ -34,6 +32,8 @@ class TextRange;
  */
 class TextBlock
 {
+    friend class TextBuffer;
+
 public:
     /**
      * Construct an empty text block.
@@ -194,7 +194,10 @@ public:
      */
     void insertCursor(Kate::TextCursor *cursor)
     {
-        m_cursors.insert(cursor);
+        auto it = std::lower_bound(m_cursors.begin(), m_cursors.end(), cursor);
+        if (it == m_cursors.end() || cursor != *it) {
+            m_cursors.insert(it, cursor);
+        }
     }
 
     /**
@@ -203,16 +206,10 @@ public:
      */
     void removeCursor(Kate::TextCursor *cursor)
     {
-        m_cursors.remove(cursor);
-    }
-
-    /**
-     * Returns the size of this block i.e.,
-     * the count of QChars it has + number of new lines
-     */
-    int blockSize() const
-    {
-        return m_blockSize + m_lines.size();
+        auto it = std::lower_bound(m_cursors.begin(), m_cursors.end(), cursor);
+        if (it != m_cursors.end() && cursor == *it) {
+            m_cursors.erase(it);
+        }
     }
 
 private:
@@ -232,16 +229,10 @@ private:
      */
     std::vector<Kate::TextLine> m_lines;
 
-    /**
-     * size of block i.e., number of QChars
-     */
-    int m_blockSize = 0;
-
     /**
      * Set of cursors for this block.
-     * using QSet is better than unordered_set for perf reasons
      */
-    QSet<TextCursor *> m_cursors;
+    std::vector<TextCursor *> m_cursors;
 };
 }
 
