@@ -19,22 +19,17 @@ class KateRenderer;
 class KateLineLayoutMap
 {
 public:
-    inline void clear();
+    void clear();
 
-    inline bool contains(int i) const;
+    void insert(int realLine, std::unique_ptr<KateLineLayout> lineLayoutPtr);
 
-    inline void insert(int realLine, const KateLineLayoutPtr &lineLayoutPtr);
+    void relayoutLines(int startRealLine, int endRealLine);
 
-    inline void viewWidthIncreased();
-    inline void viewWidthDecreased(int newWidth);
+    void slotEditDone(int fromLine, int toLine, int shiftAmount, std::vector<KateTextLayout> &textLayouts);
 
-    inline void relayoutLines(int startRealLine, int endRealLine);
+    KateLineLayout *find(int i);
 
-    inline void slotEditDone(int fromLine, int toLine, int shiftAmount);
-
-    KateLineLayoutPtr &operator[](int i);
-
-    typedef QPair<int, KateLineLayoutPtr> LineLayoutPair;
+    typedef std::pair<int, std::unique_ptr<KateLineLayout>> LineLayoutPair;
 
 private:
     typedef std::vector<LineLayoutPair> LineLayoutMap;
@@ -58,8 +53,6 @@ private:
 
 class KateLayoutCache : public QObject
 {
-    Q_OBJECT
-
 public:
     explicit KateLayoutCache(KateRenderer *renderer, QObject *parent);
 
@@ -71,7 +64,7 @@ public:
     bool wrap() const;
     void setWrap(bool wrap);
 
-    bool acceptDirtyLayouts();
+    bool acceptDirtyLayouts() const;
     void setAcceptDirtyLayouts(bool accept);
 
     // BEGIN generic methods to get/set layouts
@@ -83,13 +76,13 @@ public:
      * cache is full or until they are invalidated by other means (eg. the text
      * changes).
      *
+     * This function shall never return null
+     *
      * \param realLine real line number of the layout to retrieve.
      * \param virtualLine virtual line number. only needed if you think it may have changed
      *                    (ie. basically internal to KateLayoutCache)
      */
-    KateLineLayoutPtr line(int realLine, int virtualLine = -1);
-    /// \overload
-    KateLineLayoutPtr line(const KTextEditor::Cursor realCursor);
+    KateLineLayout *line(int realLine, int virtualLine = -1);
 
     /// Returns the layout describing the text line which is occupied by \p realCursor.
     KateTextLayout textLayout(const KTextEditor::Cursor realCursor);
@@ -135,11 +128,11 @@ public:
     void viewCacheDebugOutput() const;
     // END
 
-private Q_SLOTS:
-    void wrapLine(const KTextEditor::Cursor position);
-    void unwrapLine(int line);
-    void insertText(const KTextEditor::Cursor position, const QString &text);
-    void removeText(KTextEditor::Range range);
+private:
+    void wrapLine(KTextEditor::Document *, const KTextEditor::Cursor position);
+    void unwrapLine(KTextEditor::Document *, int line);
+    void insertText(KTextEditor::Document *, const KTextEditor::Cursor position, const QString &text);
+    void removeText(KTextEditor::Document *, KTextEditor::Range range, const QString &);
 
 private:
     KateRenderer *m_renderer;
@@ -150,16 +143,16 @@ private:
      * Layouts which are not within the current view cache and whose
      * refcount == 1 are only known to the cache and can be safely deleted.
      */
-    mutable KateLineLayoutMap m_lineLayouts;
+    KateLineLayoutMap m_lineLayouts;
 
     // Convenience vector for quick direct access to the specific text layout
-    KTextEditor::Cursor m_startPos;
+    KTextEditor::Cursor m_startPos = KTextEditor::Cursor::invalid();
 
-    mutable std::vector<KateTextLayout> m_textLayouts;
+    std::vector<KateTextLayout> m_textLayouts;
 
-    int m_viewWidth;
-    bool m_wrap;
-    bool m_acceptDirtyLayouts;
+    int m_viewWidth = 0;
+    bool m_wrap = false;
+    bool m_acceptDirtyLayouts = false;
 };
 
 #endif
