@@ -228,7 +228,12 @@ public:
 
         friend bool operator==(const SecondaryCursor &l, const SecondaryCursor &r)
         {
-            return l.cursor() == r.cursor();
+            return l.cursor() == r.cursor() && l.selectionRange() == r.selectionRange();
+        }
+
+        KTextEditor::Range selectionRange() const
+        {
+            return range ? range->toRange() : KTextEditor::Range::invalid();
         }
 
         void clearSelection()
@@ -980,9 +985,6 @@ public:
 public:
     KTextEditor::Range visibleRange();
 
-Q_SIGNALS:
-    void displayRangeChanged(KTextEditor::ViewPrivate *view);
-
 protected:
     bool event(QEvent *e) override;
 
