@@ -207,6 +207,7 @@ public:
     Range motionRepeatlastTFBackward();
 
     Range motionToEOL();
+    Range motionToLastNonBlank();
     Range motionToColumn0();
     Range motionToFirstCharacterOfLine();
 
@@ -334,10 +335,17 @@ protected:
 protected:
     // The 'current position' is the current cursor position for non-linewise pastes, and the current
     // line for linewise.
-    enum PasteLocation { AtCurrentPosition, AfterCurrentPosition };
+    enum PasteLocation {
+        AtCurrentPosition,
+        AfterCurrentPosition
+    };
     bool paste(NormalViMode::PasteLocation pasteLocation, bool isgPaste, bool isIndentedPaste);
     static KTextEditor::Cursor cursorPosAtEndOfPaste(const KTextEditor::Cursor pasteLocation, const QString &pastedText);
 
+    // set sticky column to a ridiculously high value so that the cursor will stick to EOL,
+    // but only if it's a regular motion
+    void stickStickyColumnToEOL();
+
 protected:
     QString m_keys;
     QString m_lastTFcommand; // holds the last t/T/f/F command so that it can be repeated with ;/,
